<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of Plugin "Widget Delicious".
# Copyright (c) 2008 Pascal MARTIN and contributors. All rights
# reserved.
#
# Plugin "Widget Delicious" is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Plugin "Widget Delicious" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with DotClear; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# ***** END LICENSE BLOCK *****

class publicWidgetDelicious
{
    public static function delicious(&$w)
    {
        return self::_getContent($w);
    }
    
    
    protected static $defaultCacheFile = 'delicious.cache.html';
    protected static $defaultIdDiv = 'delicious';
    protected static $defaultClassDiv = 'delicious';
    protected static $defaultTitle = 'Mes favoris del.icio.us';
    protected static $defaultIdAllLinks = 'allDeliciousLinks';
    protected static $defaultTextAllLinks = 'Tous mes favoris sur del.icio.us';
    
    protected static $minTimeout = 2;
    
    
    protected static function _getContent($w)
    {
        $cacheFile = trim($w->cacheFile);
        if ($cacheFile === '')
        {
            $cacheFile = self::$defaultCacheFile;
        }
        $cacheFile = realpath(DC_TPL_CACHE) . DIRECTORY_SEPARATOR . $w->cacheFile;
        $cacheLifetime = intval($w->cacheDuration)*60;
        
        clearstatcache();
        
        if (file_exists($cacheFile)
            && (@filemtime($cacheFile) > time() - $cacheLifetime))
        {
            $file = fopen($cacheFile, 'r');
            if ($file)
            {
                if (flock($file, LOCK_SH))
                {
                    $toDisplay = fread($file, filesize($cacheFile)+4);
                    fclose($file);
                    return $toDisplay;
                }
                fclose($file);
                return '';
            }
        }
        
        $file = fopen($cacheFile, 'w+');
        if ($file)
        {
            if (flock($file, LOCK_EX))
            {
                ob_start();
                self::_displayContentFromDelicious($w, $file);
                $toDisplay = ob_get_contents();
                ob_end_clean();
                
                ftruncate($file, 0);
                fwrite($file, $toDisplay);
                fclose($file);
                
                return $toDisplay;
            }
            fclose($file);
        } // Fin si fichier a pu être ouvert
        
        return '';
    } // _getContent
    
    
    
    protected static function _getXMLFromDelicious($w, $nbrLinksToFetch, & $errorCode, & $errorMsg)
    {
        $login = trim($w->login);
        $password = trim($w->password);
        
        if ($login === '' || $password === '')
        {
            return null;
        }
        
        $timeout = intval($w->timeout);
        if ($timeout < self::$minTimeout)
        {
            $timeout = self::$minTimeout;
        }
        
        if (function_exists('curl_init'))
        {
            $url = 'https://api.del.icio.us/v1/posts/recent?count=' . urlencode($nbrLinksToFetch);
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_HTTPGET, true);
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC | CURLAUTH_DIGEST);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
            curl_setopt($ch, CURLOPT_USERPWD, $login . ':' . $password);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            
            $stringXMLData = curl_exec($ch);
            if ($stringXMLData !== false)
            {
                $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                if ($httpCode >= 200 && $httpCode < 300)
                {
                    curl_close($ch);
                    return simplexml_load_string($stringXMLData);
                }
            }
            else
            {
                $errorCode = curl_errno($ch);
                $errorMsg = curl_error($ch);
                $errorMsg = str_replace(array($login, $password), array('LOGIN', 'PASSWORD'), $errorMsg);
            }
            curl_close($ch);
            return null;
        }
        else if (ini_get('allow_url_fopen')
            && in_array('https', stream_get_wrappers()))
        {
            $oldTimeout = ini_get('default_socket_timeout');
            $url = 'https://' . urlencode($login) . ':' . urlencode($password) . 'api.del.icio.us/v1/posts/recent?count=' . urlencode($nbrLinksToFetch);
            ini_set('default_socket_timeout', $timeout);
            $stringXMLData = @file_get_contents($url);
            ini_set('default_socket_timeout', $oldTimeout);
            if ($stringXMLData !== false)
            {
                return simplexml_load_string($stringXMLData);
            }
            return null;
        }
        else
        {
            $errorCode = -1;
            $errorMsg = "Extension 'curl' non disponible ; 'allow_url_fopen' à 'Off' ou wrapper 'https' non activé.";
            //$errorMsg .= "\n" . var_export(ini_get('allow_url_fopen'), true);
            //$errorMsg .= "\n" . var_export(stream_get_wrappers(), true);
            //$errorMsg .= "\n" . var_export(stream_get_transports(), true);
        }
        
        return null;
    } // _getXMLFromDelicious
    
    
    protected static function _displayContentFromDelicious($w, $file)
    {
        $idDiv = trim($w->idDiv);
        if ($idDiv === '')
        {
            $idDiv = self::$defaultIdDiv;
        }
        $classDiv = trim($w->classDiv);
        if ($classDiv === '')
        {
            $classDiv = self::$defaultClassDiv;
        }
        $title = trim($w->title);
        if ($title === '')
        {
            $title = self::$defaultTitle;
        }
        
        $idDivEscaped = htmlspecialchars($idDiv);
        $classDivEscaped = htmlspecialchars($classDiv);
        
        $tagsAutorises = preg_split('/[\s,;]+/', $w->tagsYes, -1, PREG_SPLIT_NO_EMPTY);
        $tagsInterdits = preg_split('/[\s,;]+/', $w->tagsNo, -1, PREG_SPLIT_NO_EMPTY);
        
        $nbrLinksToDisplay = intval($w->numLinksToDisplay);
        if ($nbrLinksToDisplay === 0)
        {
            $nbrLinksToDisplay = 10;
        }
        $nbrLinksToFetch = $nbrLinksToDisplay + mt_rand(0, 15);
        
        $errorCode = 0;
        $errorMsg = null;
        $xml = self::_getXMLFromDelicious($w, $nbrLinksToFetch, $errorCode, $errorMsg);
        if ($xml !== null)
        {
            date_default_timezone_set('Europe/Paris');
            
            $postsGroupByDate = array();
            $nbrPostsOK = 0;
            foreach ($xml->post as $post)
            {
                $postAttributes = $post->attributes();
                if (!isset($postAttributes['shared']) || (string)$postAttributes['shared']!=='no')
                {
                    $tagsPost = explode(' ', (string)$postAttributes['tag']);
                    $tagsOK = array_intersect($tagsAutorises, $tagsPost);
                    $tagsNotOK = array_intersect($tagsInterdits, $tagsPost);
                    if (!empty($tagsOK)
                        && empty($tagsNotOK))
                    {
                        $date = date('d/m/Y', strtotime($postAttributes['time']));
                        $postsGroupByDate[$date][] = (object)array(
                                    'href' => (string)$postAttributes['href'],
                                    'description' => (string)$postAttributes['description'],
                                    'tags' => $tagsOK
                                );
                        if (++$nbrPostsOK >= $nbrLinksToDisplay)
                        {
                            break;
                        }
                    }
                }
            }
            
            if (!empty($postsGroupByDate))
            { // On a des favoris à afficher
                echo '<div id="' . $idDivEscaped . '" class="' . $classDivEscaped . '">';
                echo '<h2>' . htmlspecialchars($title) . '</h2>';
                
                if (!intval($w->displayDates))
                {
                    echo '<ul>';
                }
                
                foreach ($postsGroupByDate as $date => $posts)
                {
                    if (intval($w->displayDates))
                    {
                        echo '<h3>' . $date . '</h3>';
                        echo '<ul>';
                    }
                    
                    foreach ($posts as $post)
                    {
                        $hrefPost = $post->href;
                        $descriptionPost = $post->description;
                        $tagsPost = $post->tags;
                        
                        foreach ($tagsPost as & $tag)
                        {
                            $tag = htmlspecialchars(strip_tags($tag));
                        }
                        $descriptionPost = htmlspecialchars(strip_tags($descriptionPost));
                        
                        echo '<li>';
                        if (intval($w->useFavicons))
                        {
                            $urlFavicon = implode('/', array_slice(explode('/', $hrefPost), 0, 3)) . '/favicon.ico';
                            echo '<span style="width: 16px; height: 16px;"><img src="' . htmlspecialchars($urlFavicon) . '" alt="" style="border: none; display: inline; position: absolute; width: 16px; height: 16px;" /></span>';
                            echo '<a href="' . htmlspecialchars($hrefPost) . '" title="Tags : ' . implode(', ', $tagsPost) . '" style="margin-left: 20px;">' . $descriptionPost . '</a>';
                        }
                        else
                        {
                            echo '<a href="' . htmlspecialchars($hrefPost) . '" title="Tags : ' . implode(', ', $tagsPost) . '">' . $descriptionPost . '</a>';
                        }
                        echo '</li>';
                    }
                    
                    if (intval($w->displayDates))
                    {
                        echo '</ul>';
                    }
                }
                
                if (!intval($w->displayDates))
                {
                    echo '</ul>';
                }
                
                
                if (intval($w->displayAllLink))
                {
                    
                    $identifiantLien = trim($w->idAllLink);
                    if ($identifiantLien === '')
                    {
                        $identifiantLien = self::$defaultIdLinkAll;
                    }
                    $textAllLinks = trim($w->textAllLinks);
                    if ($textAllLinks === '')
                    {
                        $textAllLinks = self::$defaultTextAllLinks;
                    }
                    echo '<div id="' . htmlspecialchars($identifiantLien) . '"><a href="http://del.icio.us/' . htmlspecialchars(urlencode($w->login)) . '">' . htmlspecialchars($textAllLinks) . '</a></div>';
                }
                
                if (intval($w->useFavicons))
                {
                    $defaultFavicon = trim($w->defaultFavicon);
                    if ($defaultFavicon === '')
                    {
                        $defaultFavicon = 'null';
                    }
                    else
                    {
                        $defaultFavicon = "'" . htmlspecialchars($defaultFavicon) . "'";
                    }
                    echo <<<WIDGET_DELICIOUS_FAVICONS
<script type="text/javascript">
(function () {
    var d = document.getElementById('{$idDivEscaped}');
    if (d !== undefined) {
        var gestionImg = function (img) {
            img.onerror=function(){
                img.src={$defaultFavicon};
                img.onerror=function(){};
            };
        };
        var uls = d.getElementsByTagName('ul');
        for (var i_ul=0 ; i_ul<uls.length ; i_ul++) {
            var lis = uls[i_ul].getElementsByTagName('li');
            for (var i_li=0 ; i_li<lis.length ; i_li++) {
                var imgs = lis[i_li].getElementsByTagName('img');
                if (imgs[0] !== undefined) {
                    gestionImg(imgs[0]);
                }
            }
        }
    }
})();
</script>
WIDGET_DELICIOUS_FAVICONS;
                }
                
                echo '</div>';
            } // Fin de s'il y a des favoris à afficher
        } // Fin si données XML reçues de del.icio.us
        else
        { // On n'a rien reçu de del.icio.us...
            // Pas grand chose à faire... poser le code erreur en commentaire HTML, des fois que le webmaster veuille un retour ?
            if (intval($w->errorAsHTMLComment))
            {
                echo '<!-- ' . htmlspecialchars($errorCode) . ' - ' . htmlspecialchars($errorMsg) . ' -->';
            }
        } // Fin si rien reçu de del.icio.us
    } // _displayContentFromDelicious
    
    
    
} // class publicWidgetDelicious
?>